from distutils.core import setup
setup(
  name='elavonvtpv',
  packages=['elavonvtpv'],
  version='1.4.5',
  description='Python 3 package to comunicate with the Elavon Virtual TPV Remote API',
  author='Javier Ros Honduvilla',
  author_email='javier.ros.honduvilla@gmail.com',
  url='https://github.com/jros99/elavontpv',
  download_url='https://github.com/jros99/elavontpv/tarball/0.3',
  keywords=['elavon', 'tpv', 'virtual tpv'],
  classifiers=[],
)